from console_base.console_keyring import ConsoleKeyring
from invoke import call, task
from lcconfig import (
    ConsoleSettingsConfig,
    ConsoleImpersonatorConfig,
)
from pathlib import Path
try:
    from psycopg import connect as db_connect
except ModuleNotFoundError:
    from psycopg2 import connect as db_connect

from .sys import restart_webserver, restart
from logcabinctl.settings import (
    CONSOLE_HOME,
    CONF_DIR,
    DB_HOST,
    IS_DRAWBRIDGE_OS,
    PYTHON,
    PORTAL_CODE_NAME,
)


@task(post=[restart])
def update(ctx):
    """
    Run upgrades script(s)
    """
    print('Running console upgrade script(s)')
    if IS_DRAWBRIDGE_OS:
        ctx.sudo(f'{PYTHON} {CONSOLE_HOME}/logcabin_upgrades.py', pty=True, user='portal')
        ctx.run('update portal static', pty=True)
        ctx.run('update redwood patterns', pty=True)
    else:
        ctx.run(f'{PYTHON} {CONSOLE_HOME}/logcabin_upgrades.py', pty=True)
        ctx.run('/usr/bin/bash /opt/sap/scripts/rulesupdater.sh', pty=True)
        ctx.run('/usr/bin/bash /opt/sap/scripts/staticupdater.sh', pty=True)


@task(post=[restart_webserver], help={'force': 'Refresh TLS certificate whether stale or not'})
def refresh_cert(ctx, force=False):
    """
    Refresh Console web server TLS certificate
    """
    print('Refresh Console TLS certificate')
    cmd = f'{PYTHON} {CONSOLE_HOME}/manage.py update_console_cert'

    if force:
        cmd = f'{cmd} --force'

    if IS_DRAWBRIDGE_OS:
        ctx.sudo(cmd, pty=True, user='portal')
    else:
        ctx.run(cmd, pty=True)


@task(help={'force': 'Refresh Redwood CA whether stale or not'})
def refresh_redwood_ca(ctx, force=False):
    """
    Refresh Console web server TLS certificate
    """
    print('Refresh Redwood certificate authority')
    cmd = f'{PYTHON} {CONSOLE_HOME}/manage.py renew_redwood_ca'

    if force:
        cmd = f'{cmd} --force'

    if IS_DRAWBRIDGE_OS:
        ctx.sudo(cmd, pty=True, user='portal')
    else:
        ctx.run(cmd, pty=True)


@task(help={'force': 'Reload configuration files whether stale or not'})
def reload_config(ctx, force=False):
    """
    Regenerate Console and Redwood configuration files
    """
    print('Regenerate Console and Redwood configuration files')
    cmd = f'{PYTHON} {CONSOLE_HOME}/manage.py reload_config'

    if force:
        cmd = f'{cmd} --force'

    if IS_DRAWBRIDGE_OS:
        ctx.sudo(cmd, pty=True, user='portal')
    else:
        ctx.run(cmd, pty=True)


@task(
    pre=[update, reload_config],
    help={'force': 'Refresh Console & Redwood CA certificates whether stale or not'},
)
def deploy(ctx, force=False):  # noqa
    """
    Run upgrade script(s) and refresh webserver certificate
    """
    call(refresh_cert, force=force),
    call(refresh_redwood_ca, force=force),
    print('Console updated and deployed')


@task
def reset(ctx):
    """
    Reset console by deleting database and resetting config files.
    To restore functionality, it will be necessary to re-register,
    or impersonate another appliance.
    """
    from .cache import flush
    flush(ctx)

    if IS_DRAWBRIDGE_OS:
        name = 'DrawBridge Portal'
        database = 'drawbridge'
    else:
        name = 'Log Cabin Console'
        database = 'logcabin'

    print(f'\nResetting the {name} will delete the database and clear config files!\n')
    proceed = input('Are you sure you want to continue? [no/yes] ')

    if proceed.lower() == 'no':
        raise SystemExit(f'\nExiting {name} Reset\n')

    if proceed.lower() not in ('no', 'yes'):
        raise SystemExit(f'\n{proceed} is an invalid value\n')

    print('Resetting settings.conf')
    impersonator_data = ConsoleImpersonatorConfig().as_typed_dict('BoxData')
    console_cfg = ConsoleSettingsConfig()
    console_data = console_cfg.as_typed_dict('BoxData')
    console_data.update(impersonator_data)
    console_cfg.save_section('BoxData', console_data)

    print(f'Deleting {name} Database: {database}')

    try:
        conn = db_connect(f"user='{PORTAL_CODE_NAME}' host={DB_HOST} port=5432")
        conn.autocommit = True
        cursor = conn.cursor()
        cursor.execute(f'DROP DATABASE IF EXISTS {database}')
    except Exception:
        raise SystemExit(f'Unable to drop {database}. Server may not be running.')

    if conn.closed == 0:
        conn.close()

    print(f'Clear Console Keyring')
    kr = ConsoleKeyring()
    Path(kr.file_path).unlink(missing_ok=True)

    for config_file in (
            f'{CONF_DIR}/console.key',
            f'{CONF_DIR}/console.pem',
    ):
        Path(config_file).unlink(missing_ok=True)


__all__ = (
    'deploy',
    'refresh_cert',
    'reload_config',
    'refresh_redwood_ca',
    'update',
    'reset',
)
